package jexplorer;

/* Autor:           Dipl.-Inf., Dipl.-Ing. (FH) Michael Wilhelm
   Firma:           HS Harz, FB AI
   EMail:           mwilhelm@hs-harz.de
   Projekt:         JExplorer
   Version:         0.8
   Ersterstellung:  10.10.2006
   Letzte nderung: 25.11.2006
*/

// Klassen importieren
import java.awt.*;
import java.io.File;
import javax.swing.*;
import javax.swing.tree.*;
import javax.swing.event.*;
import javax.swing.plaf.*;


// Klasse TreeFeld, als JPanel programmiert
// Der Baum ist IM Panel !!!, Komposition 
public class CTree extends JPanel implements  TreeSelectionListener, TreeExpansionListener {
	// Ansicht des Baumes
  	JTree DirTree;
  	// Rollfeld fuer den Baum
  	// JScrollPane scrollpane;  // JTree braucht KEINE ScrollPane
        
  	// Referenz auf das Explorer-Fenster
  	IExplorer iex; 
        
        DefaultTreeModel treeModel;

        
  	// Konstruktor
	public CTree(IExplorer iexPar, String sRoot) {
    	// Referenz auf das Explorer-Fenster, der Explorer fuehrt das FocusDirectory mit
            this.iex = iexPar; 
              
            setLayout(new BorderLayout());            
            setTree(sRoot);
      	}//TreeFeld

        
        
    
        // setzt die Wurzel
        // Aufgerufen aus JExplorer, JToolbar
        private void setTree(String sRootName) {
            CDefaultMutableTreeNode rootNode;        
            rootNode = BuildTree( sRootName  );

            treeModel = new DefaultTreeModel(rootNode);
            DirTree = new javax.swing.JTree(rootNode); // treeModel);  //  kein Root !!!
            
            add(DirTree, BorderLayout.CENTER);
            
            DirTree.getSelectionModel().setSelectionMode(TreeSelectionModel.SINGLE_TREE_SELECTION);
            
            DirTree.getSelectionModel().setSelectionMode(TreeSelectionModel.SINGLE_TREE_SELECTION);
            DirTree.setBorder(BorderFactory.createLineBorder(Color.black));
            DirTree.setToolTipText("ToolTipp");
            DirTree.setShowsRootHandles(true);

              // Horizontale Linien
            DirTree.putClientProperty("JTree.lineStyle","Horizontal");
              // Mit Linien zwischen den Knoten
            DirTree.putClientProperty("JTree.lineStyle","Angled");


            DirTree.getSelectionModel().setSelectionMode(TreeSelectionModel.SINGLE_TREE_SELECTION);
            
            // Listener hinzufuegen
            DirTree.addTreeSelectionListener(this);
            DirTree.addTreeExpansionListener(this);
            
            DirTree.putClientProperty("JTree.lineStyle","Angled");
            DirTree.setShowsRootHandles(true);
            DirTree.setSelectionRow(0);
            
            DirTree.setFont( new Font("Arial", Font.BOLD,18)  );
            
        }
        
        // Diese Methode wird von der Tabelle MKDIR aufgerufen
        // Der aktuelle Knoten wird neugeladen
        // Grund:  Anlegen eines neuen Verzeichnisses
        public void reloadNode() {
            CDefaultMutableTreeNode node;
            CTreeNode myNode;
            node = (CDefaultMutableTreeNode) DirTree.getLastSelectedPathComponent();
            
            if (node == null) {
                CBasis.Error("Fehler","node==null");
                return;
            }

            myNode = (CTreeNode) node.getUserObject();
            if ( myNode.sPath!="" ) {
                // System.out.println(myNode.sPath);
                // Nun alle Unterknoten lschen, kommen eh wieder
                node.removeAllChildren();
                  AddSubDirs(node);
            }
            else {
                // Diese Fehlermeldung sollte nie erscheinen
                CBasis.Error("AddsubDires","Diese Fehlermeldung sollte nie erscheinen\n"+
                           "Methode: reloadNode, sPath == null");
            }
            
            //CDefaultMutableTreeNode root;
            // Scan(root, sPath, sPath, 0);
        }
        
        
        
        public void setRoot(String sRootName ) {
          	// Entfernen des alten Baumes; removeAll entfernt alle angehngten Components von JPanel
            removeAll();                      
            setTree(sRootName);
            updateUI();
        }
        
        
   CDefaultMutableTreeNode BuildTree(String sPath) {
        CDefaultMutableTreeNode root;

        root = createMyNode(  sPath, sPath);  // new DefaultMutableTreeNode("root");
        Scan(root, sPath, sPath, 0);
        return root;
      }  // BuildTree


   // rekursives Lesen des Baumes, aber nur zwei Stufen tief
   // dies ist notwendig, um das "+"-Zeichen im Baum zu bekommen
    void  Scan(CDefaultMutableTreeNode parent, String sPath, String sName, int Stufe) {
        int i;
        CDefaultMutableTreeNode node;
        int AnzSubDir;
        CTreeNode myNode;

        File dir = new File( sPath );
        File[] items = dir.listFiles();
        if (items==null) {
          return;
        }

        if (Stufe>1) {
            AnzSubDir=0;
            for (i=0; i<items.length; i++) {
               if ( items[i].isDirectory()  && (!items[i].isHidden())  ) {
                    AnzSubDir++;
               }
            }
            parent.setAnzSubdir(AnzSubDir);
            return;
        }
        
        
        if ( items != null ) {
            for (i=0; i<items.length; i++) {
               if ( items[i].isDirectory()  && (!items[i].isHidden()) ) {                    
                   String s = items[i].getName() ;
                    node = createMyNode( sPath+items[i].getName()+"\\",   items[i].getName()  );  // new DefaultMutableTreeNode("root");
                    parent.add(node);  
                    //      System.out.println(items[i].getName() );
                    Scan(node, sPath+items[i].getName()+"\\",  items[i].getName(), Stufe+1  ) ;
                    
               }
            }
        }
    } // Scan


    
    
    CDefaultMutableTreeNode createMyNode(String sPath,  String nodeString) {
        CTreeNode myNode;
        CDefaultMutableTreeNode Node;

        myNode = new  CTreeNode(sPath, nodeString);
        Node = new CDefaultMutableTreeNode(myNode);
        // Node = new DefaultMutableTreeNode(nodeString);
        //Node.setUserObject(myNode);
    return Node;
 }
         
    
    
	//TreeSelectionListener implementieren bei Aenderungen des Unterbaums
	public void valueChanged(TreeSelectionEvent e) {
            CDefaultMutableTreeNode node;
            CTreeNode myNode;
            node = (CDefaultMutableTreeNode) DirTree.getLastSelectedPathComponent();
            
            if (node == null) return;

              myNode = (CTreeNode) node.getUserObject();
              // System.out.println("valueChanged: "+node.toString()+" \n" +myNode.sPath );
              if ( myNode.sPath!="" ) {
                      // Aufruf und Erzeugen eines Dialogfensters
                  // hat der Knoten doch SubNodes, obwohl noch nicht verknpft ?
                  if (  (node.getChildCount()==0)  &&  (node.getAnzSubDir() > 0 )  ) {
                      // System.out.println(   node.getAnzSubDir()+"   "+node.getChildCount()   );
                      AddSubDirs(node);
                  }
                  else {
                      // System.out.println(   "keine: "+node.getAnzSubDir()+"   "+node.getChildCount()  );
                      
                  }
                  iex.setTableDir( myNode.sPath  );
              }
              else {
                System.out.println("keine Auswahl");
              }
            
            /*
             //Unterbaum einlesen oder nicht einlesen je nach Auswahl			
            if (node.getChildCount() < 1) {
                    DirTree.addSubTree(node,node.getString(),0);
                    treeView.validate();
            }//if
            // Focus auf ausgewaehlten Knoten setzen
            iex.setFocusNode((DefaultMutableTreeNode) node);
            Object[] pathStrArr = node.getUserObjectPath();
            String pathStr = node.getString();
            if (!(pathStr == null)) {
                    iex.setTitle(pathStr);
                    iex.setTableDir(pathStr);
            }//if
             */
  	}// valueChanged
        
        
        	// Wird aufgerufen, wenn ein Eintrag im Baum kollabiert
	public void treeCollapsed(TreeExpansionEvent event){
	}//treeCollapsed

	// Erweiterung des Baumes
	public void treeExpanded(TreeExpansionEvent event){  
  	}//treeExpanded
  
        private void AddSubDirs(CDefaultMutableTreeNode node) {
            CTreeNode myNode;
            myNode = (CTreeNode) node.getUserObject();
            Scan(node, myNode.sPath, node.toString(), 0);            
        }

        
    public void collapse() {
        DirTree.collapseRow(0);
    }
    
    public  void expand() {
        for (int i=0; i<DirTree.getRowCount(); i++) {
          DirTree.expandRow(i);
        }
    }
        
}


